#include <MainWindow.h>
#include <ui_MainWindow.h>
#include <QTimer>
#include <QPaintEvent>
#include <QPainter>
#include <QMatrix>
#include <math.h>
#include <QDebug>
#include <QVector>
#include "CRingBuff.h"

MainWindow::~MainWindow()
{
	delete m_Buffer;
}

MainWindow::MainWindow()
{
    setupUi(this);
    connect(StartB, SIGNAL(clicked()), this, SLOT(Start()));
    connect(StopB, SIGNAL(clicked()), this, SLOT(Stop()));
    connect(ClearB, SIGNAL(clicked()), this, SLOT(Clear()));
    connect(FreqXSB, SIGNAL(valueChanged(int)), this, SLOT(SetFreqX(int)));
    connect(FreqYSB, SIGNAL(valueChanged(int)), this, SLOT(SetFreqY(int)));
    connect(AmpXSB, SIGNAL(valueChanged(int)), this, SLOT(SetAmpX(int)));
    connect(AmpYSB, SIGNAL(valueChanged(int)), this, SLOT(SetAmpY(int)));
    connect(PhaseXSB, SIGNAL(valueChanged(int)), this, SLOT(SetPhaseX(int)));
    connect(PhaseYSB, SIGNAL(valueChanged(int)), this, SLOT(SetPhaseY(int)));
    timer = new QTimer(this);
	
	m_t=0;
	m_Buffer=new CRingBuff(100);
	
    //LastPTS = new QPointF(300)
//    TraceBufferX.push_front(0);
//    TraceBufferY.push_front(0);
//    TraceBufferX.push_front(0);
//    TraceBufferY.push_front(0);
	/*
    for(i=0; i<255; i++)
    {
        TraceBufferX.push_front(0);
        TraceBufferY.push_front(0);
    }
	*/
    R1=100;
    R2=100;
    w1=1;
    w2=1;
    f1=0;
    f2=0;
    j=0;
	
    FreqXSB->setRange(0,1000);
	FreqXSB->setValue(3);
	SetFreqX(FreqXSB->value());
			
    FreqYSB->setRange(0,1000);
	FreqYSB->setValue(1);
    SetFreqY(FreqYSB->value());
	
    AmpXSB->setRange(0,250);
    AmpYSB->setRange(0,250);
	
    PhaseXSB->setRange(1,1000);
	PhaseXSB->setValue(0);
	SetPhaseX(PhaseXSB->value());
	
    PhaseYSB->setRange(0,1000);

    AmpXSB->setValue(100);
    AmpYSB->setValue(100);
}

void MainWindow::Start()
{
    //TraceBufferX.pop_front();
    //TraceBufferY.pop_front();
    connect(timer, SIGNAL(timeout()), this, SLOT(NewFrame()));
    timer->start(10);
}

void MainWindow::Stop()
{
    timer->stop();
}

void MainWindow::Clear()
{
    Stop();
    update();
}

void MainWindow::SetFreqX(int CurrentFreqX)
{
    w1=CurrentFreqX;
    w0=(w1+w2)/2;
}

void MainWindow::SetFreqY(int CurrentFreqY)
{
    w2=CurrentFreqY;
    w0=(w1+w2)/2;
}

void MainWindow::SetAmpX(int CurrentAmpX)
{
    R1=CurrentAmpX;
}

void MainWindow::SetAmpY(int CurrentAmpY)
{
    R2=CurrentAmpY;
}

void MainWindow::SetPhaseX(int CurrentPhaseX)
{
    f1=CurrentPhaseX;
}

void MainWindow::SetPhaseY(int CurrentPhaseY)
{
    f2=CurrentPhaseY;
}

void MainWindow::NewFrame()
{
	QRect winR=geometry();
	winR.moveTopLeft(QPoint(0,0));
	
	int x0=winR.width()/2;
	int y0=winR.height()/2;
	
	
	
	
	int x=R1*cos(w1*(0-0.05*m_t)+f1)+x0;
	int y=R2*sin(w2*(0-0.05*m_t)+f2)+y0;
	
	
	m_Buffer->push(QPoint(x,y));
	
/*    TraceBufferX.pop_back();
    TraceBufferY.pop_back();
    TraceBufferX.push_front();
    TraceBufferY.push_front();
	*/
//    if(TraceBufferX.size()==255)
//    {
//        TraceBufferX.pop_back();
//        TraceBufferY.pop_back();
//    }
    m_t+=3;//dt;
    update();
}

void MainWindow::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
	
	QRect winR=geometry();
	winR.moveTopLeft(QPoint(0,0));
	
    painter.fillRect(winR,QColor(100,130,100));
	
//    painter.setMatrix(QMatrix(1,0,0,-1,255,255));
 //   painter.setRenderHint(QPainter::Antialiasing, true);
    //painter.setPen(QPen(QColor(255,0,0), 3, Qt::SolidLine, Qt::RoundCap));
    //painter.drawPoint(R1*cos(w1*t), R2*sin(w2*t));
    //painter.drawPoint(R1*cos(w1*(t-dt)), R2*sin(w2*(t-dt)));
    //painter.drawPoint(R1*cos(w1*(t-2*dt)), R2*sin(w2*(t-2*dt)));
	int x0=winR.width()/2;
	int y0=winR.height()/2;
	
	painter.setPen(QPen(QColor(0,255,0),2));
	m_Buffer->PaintIt(painter);
    
	/*for(i=0; i<int(1.5*w0*w0); i++)
    {
		
//        painter.setPen(QPen(QColor(255,0,0), 3, Qt::SolidLine, Qt::RoundCap));
        painter.drawPoint(
				R1*cos(w1*(t-0.05*i)+f1)+x0,
				R2*sin(w2*(t-0.05*i)+f2)+y0
				); // , ,  .
//        painter.drawPoint(10,10); // , ,  .
        //painter.drawLine(TraceBufferX.value(i), TraceBufferY.value(i), TraceBufferX.value(i-1), TraceBufferY.value(i-1)); //  
        //painter.drawPoint(TraceBufferX.value(i), TraceBufferY.value(i)); //  ,  .
    }*/
}

